/**
 * 组件打包webpack配置
 * @author yanrong.tian
 * @date 2020/5/16 14:22:00
 */
process.env.NODE_ENV = 'production';
const Webpack = require('webpack');
const chalk = require('chalk');
const { getWebpackConfig } = require('./pack.config.js');
var path = require('path');
const projectRoot = process.cwd();
const outputPath = path.resolve(projectRoot, 'libs');
const ProgressBarPlugin = require('progress-bar-webpack-plugin');
const { CleanWebpackPlugin } = require('clean-webpack-plugin');

// 获取命令中的config文件路径
const argv = process.argv;
let configFile = 'runner.config.js';
argv.map(item => {
  if (item.indexOf('config=') >= 0) {
    configFile = item.replace('config=', '');
  }
});
let runnerConfigPath = '';
try{
  runnerConfigPath = require.resolve(path.resolve(projectRoot, 'node_modules',configFile));
}
catch(e) {
  try{
    runnerConfigPath = require.resolve(path.resolve(projectRoot, configFile));
  }
  catch(e) {

  }
}

// 加载config文件
const runnerConfig = require(runnerConfigPath);
const layoutRoot = runnerConfig.layoutRoot || path.join(__dirname, '../template/release/src');

// 根据配置文件，获取webpack配置信息
const projectConfig = getWebpackConfig({
  command: 'pack',
  layoutRoot,
  projectRoot,
  entry: runnerConfig.packEntry || {
    index: path.resolve(layoutRoot, 'index.js'),
  },
  publicPath: runnerConfig.publicPath,
  webpackConfig: runnerConfig.webpackConfig,
  outputPath: outputPath,
  lessVariables: runnerConfig.lessVariables || {},
  templatePath:
    runnerConfig.templatePath || path.resolve(__dirname, '../template/release/src/index.ejs'),
  requirePath: path.resolve(__dirname, '../template/release/src/utils/request.js'),
  isBundleAnalyzerPlugin: runnerConfig.isBundleAnalyzerPlugin,
});

projectConfig.plugins.splice(
  0,
  0,
  new ProgressBarPlugin({
    format: 'build [:bar] :percent (:elapsed seconds)',
    clear: false,
    width: 60,
  }),
);
projectConfig.plugins.push(new CleanWebpackPlugin());

// 打包
const compiler = Webpack(projectConfig);
compiler.run((err, stats) => {
  if (err) {
    // eslint-disable-next-line no-console
    console.error(err);
    console.log(chalk.red('BUILD FAILED'));
    return false;
  }
  if (stats.hasErrors()) {
    console.log(stats.toString('minimal'));
    console.log(chalk.red('BUILD FAILED'));
    return false;
  }
  // eslint-disable-next-line no-console
  console.log(
    '\n' +
      stats.toString({
        hash: false,
        chunks: false,
        children: false,
        cached: false,
        modules: false,
        colors: true,
      }),
  );
  console.log(chalk.green('BUILD SUCCESS'));
});
