import { PureComponent, createElement } from 'react';
import { createBrowserHistory as createHistory } from 'history';
import dva, { connect } from 'dva';
import qs from 'qs';

const myApp = dva({
  history: createHistory({
    basename: _PROCESS_ENV_BASE_
  }),
});

export const getAsyncComponent = load => {
  return class AsyncComponent extends PureComponent {
    constructor(props) {
      super(props);
      this.state = {};
    }

    loadModel = () => {
      return new Promise((resolve, reject) => {
        const { config } = this.props;
        if (config.model) {
          config.model().then(
            ({ default: model }) => {
              myApp.model(model);
              this.model = model;

              resolve(model.namespace);
            },
            () => {
              resolve(null);
            },
          );
        } else {
          resolve(null);
        }
      });
    };
    componentWillUnmount() {
      if (this.model && this.model.namespace) {
        myApp.unmodel(this.model.namespace);
      }
    }
    componentDidMount() {
      this.loadModel().then(modelName => {
        load().then(({ default: component }) => {
          let newComponent = component;
          if (modelName) {
            newComponent = connect(model => {
              return {
                model: model[modelName],
              };
            })(component);
          }
          this.setState({ component: newComponent });
        });
      });
    }
    render() {
      const { component } = this.state || {};
      const newHistory = Object.assign({}, this.props.history);
      let search = newHistory.location.search;
      if(search) {
        search = search.replace('?', '');
        search = qs.parse(search);
      }
      newHistory.location.params = search;
      return component
        ? createElement(component, {
            history: newHistory,
            app: myApp,
            basename: _PROCESS_ENV_BASE_,
          })
        : null;
    }
  };
};

export const getAsyncModel = load => {
  return load;
};

export const app = myApp;
